use anyhow::Result;

pub(crate) struct Portal<'a> {
    address: &'a str,
}

#[derive(Debug)]
pub(crate) struct SamlPrelogin {}

#[derive(Debug)]
pub(crate) struct StandardPrelogin {}

#[derive(Debug)]
pub(crate) enum Prelogin {
    Saml(SamlPrelogin),
    Standard(StandardPrelogin),
}

impl<'a> Portal<'a> {
    pub fn new(address: &'a str) -> Self {
        Self { address }
    }

    // Preform the Portal prelogin
    pub async fn prelogin(&self) -> Result<Prelogin> {
        let prelogin_url = format!("https://{}/global-protect/prelogin.esp", self.address);
        let res_xml = reqwest::get(&prelogin_url).await?.text().await?;

        println!("{}", res_xml);

        Ok(Prelogin::Standard(StandardPrelogin {}))
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_new() {
        let portal = Portal::new("vpn.example.com");
        assert_eq!(portal.address, "vpn.example.com");
    }

    #[tokio::test]
    async fn test_prelogin() {
        let portal = Portal::new("vpn.microstrategy.com");
        let prelogin = portal.prelogin().await;

        assert!(prelogin.is_ok());
    }
}
